.TH FPP 1L "18 March 1991"
.\"======================================================================
.SH "NAME"
fpp \- cpp-like reversible preprocessor filter for Fortran and
SFTRAN3 code
.\"======================================================================
.SH "SYNOPSIS"
.B fpp
[
.B +debug
] [
.B +version
] [
.BI +D name
] [
.BI +D name = value
] [
.BI +U name
]
.if n .ti +5
.if t .ti +.5i
[
.B name1=value1
] .\|.\|. [
.B namek=valuek
]
.if n .ti +5
[
.B \-\^\-
]
[
.B file1
] .\|.\|. [
.B filep
]
.\"======================================================================
.SH "OPTIONS"
Because
.B fpp
is currently implemented in the
.BR nawk (1)
language, which has its own command-line switches,
.B fpp
switches are prefixed with a
.B +
instead of a
.BR \- .
.TP 8
.B \-\^\-
The preceding word on the command line is the last switch; all
following words are to be interpreted as filenames, even if they
begin with a hyphen, or contain equal signs.
.TP
.B +debug
Turn on debugging output, which is sent to
.IR stderr .
This produces helpful intermediate output from the expression
evaluator.  Macro definitions are also displayed on
.I stderr
when they are executed.
.TP
.BR +D name
Define the symbol
.I name
to the value 1.
.TP
.BR +U name
Undefine the symbol
.IR name .
If the name is subsequently referenced, it will silently evaluate
to zero.  The existence of a definition can be checked with the
.B defined
operator, or in
.IP
.nf
.\" NB: We use bold fonts here, so we need digit-width spaces, \0
.BR "C#ifdef"  "\0\0\0name"
.BR "C#ifndef" "\0\0\0name"
.fi
.IP
statements;  see below for details.
.TP
.B +version
Display the current version number of
.B fpp
on
.IR stderr .
.PP
If no input file names are given on the command line, input
is assumed to come from
.IR stdin .
.\"======================================================================
.SH "INTRODUCTION"
.PP
.B fpp
is a preprocessor for Fortran and SFTRAN3, modelled on the
ANSI C preprocessor, but tailored for Fortran use, and for
.IR reversibility .
It may serve as a prototype for a future SFTRAN3 preprocessor
conditional and macro facility.
.PP
These conventions ensure that the preprocessing is
.IR reversible ;
the output always contains the complete input, except that some
code sections may have become comments, or vice versa.  This is
more useful than the non-reversible approach used by the C
preprocessor.
.PP
Reversible preprocessing is convenient when a master source file
must be maintained to generate multiple versions, such as for
different operating-system, compiler, and architecture
variations.  The source code for any of these can serve as the
master file to create any of the others.
.PP
The current implementation is written in
.BR nawk (1).
That language is available on several different operating
systems, in both commercial and free implementations, and serves
as a convenient prototyping language for a program such as
.BR fpp .
.\"======================================================================
.SH "LANGUAGE OVERVIEW"
.B fpp
statements, or directives, are specially-formatted Fortran
comments of the forms
.PP
.nf
.B "C#name"
.BR "C#name" "  args"
.fi
.PP
Since all
.I fpp
directives are encoded as comments, both input and output files
should be compilable without any preprocessing by
.IR fpp .
.PP
Blanks may optionally surround the initial
.B #
to permit indentation for better visibility, or to reflect
conditional statement nesting.
.PP
The first column may contain any valid Fortran comment starter:
.BR C ,
.BR c ,
or
.RB * .
.PP
Unrecognized
.B "C # word"
sequences are silently copied to the output, so as to permit the
rare case of a
.B #
in the initial text of a Fortran comment.
.PP
Preprocessor names in conditionals and definitions, or set on the
command line, consist of letters, digits, and underscores; the
first character may not be a digit.
.PP
Symbols beginning with two underscores, or an underscore and an
uppercase letter, are reserved for the local implementation; see
the
.I "PREDEFINED SYMBOLS"
section below for details.
.PP
Following standard Fortran practice, letter case is not
significant in directives, or in constants and operators in
expressions.  However, if
.BR nawk (1)
is used to implement
.BR fpp ,
letter case is significant in names that are defined.  With
.BR gawk (1),
this limitation is removed; letter case is never significant.
.PP
For portability, it is recommended that lower-case letters be
used for all directives, and upper-case letters for all defined
names; this conforms to two decades of widespread practice in the
C programming language.
.\"======================================================================
.SH "DEFINITION STATEMENTS"
Definitions of names for preprocessor conditionals may be set on
the command line:
.PP
.nf
.B "fpp\0\0\0_OS_UNIX=1\0\0\0_SUN386=1"
.fi
.PP
or in the input file text itself:
.PP
.nf
.B "C\0#define\0\0\0_OS_UNIX\0\0\01"
.B "C\0#define\0\0\0_SUN386\0\0\01"
.fi
.PP
The
.BR cc (1)-like
forms with define and undefine switches are supported;
these two invocations are equivalent:
.PP
.nf
.B "fpp\0\0\0+D_OS_UNIX\0\0\0+DWORDSIZE=32\0\0\0+U_OS_VAXVMS"
.B "fpp\0\0\0_OS_UNIX=1\0\0\0WORDSIZE=32\0\0\0_OS_VAXVMS=0"
.fi
.PP
If the value is omitted, as in
.PP
.nf
.B "fpp\0\0\0_OS_UNIX=\0\0\0_SUN386="
.fi
.PP
or
.PP
.nf
.B "C\0#define\0\0\0_OS_UNIX"
.B "C\0#define\0\0\0_SUN386"
.fi
.PP
a value of 1 is assumed.
.PP
Names can be undefined by
.PP
.nf
.BR "C\0#undef"  "\0\0name"
.BR "C\0#undefine" "\0\0name"
.fi
.PP
If the name was not already defined, the request is silently ignored.
.\"======================================================================
.SH "CONDITIONAL STATEMENTS"
The conditional statements supported are:
.PP
.nf
.BR "C#if"     "\0\0\0constant-expression"
.BR "C#ifdef"  "\0\0\0name"
.BR "C#ifndef" "\0\0\0name"
.BR "C#elseif" "\0\0\0constant-expression"
.BR "C#elif"   "\0\0\0constant-expression"
.BR "C#else"   "\0\0\0[optional comment]"
.BR "C#endif"  "\0\0\0[optional comment]"
.fi
.PP
Each
.B C#ifxxx
statement must have a matching 
.B C#endif
following it.  The two may be separated by any number
of
.B C#elseif
statements, which may be followed by a single
.B C#else
statement.
.PP
A branch of a conditional is selected when the
expression evaluates to a non-zero value; see the
.I "EXPRESSIONS" 
section below for details.
.PP
Code between these statements is preserved, but in the
unselected branches of a conditional statement, a
non-comment statement will be altered to a comment by
prefixing it with an initial
.BR C## ,
shifting the statement right by three columns.  In the
selected branch, any initial
.B C##
in columns 1 through 3 is stripped; lines without this prefix
are copied verbatim.  Because of Fortran's 72-character line
length limitation, this means that code lines may not exceed 69
characters in length inside an
.B fpp
conditional.  For example, the input
.PP
.nf
.ta (\w'MMMMMM'u)
C#if\0_OS_UNIX
C##C\0\0\0\0\0UNIX\0code
C##\0\0\0\0\0\0CALL\0GETENV(...)
C#elseif\0_OS_VAXVMS
C\0\0\0\0\0VMS\0code
\0\0\0\0\0\0	CALL\0LIB$TRNLNM(...)
C#endif
.fi
.PP
when
.B _OS_UNIX=1
produces
.PP
.nf
C#if\0_OS_UNIX
C\0\0\0\0\0	UNIX\0code
\0\0\0\0\0\0	CALL\0GETENV(...)
C#elseif\0_OS_VAXVMS
C##C\0\0\0\0\0VMS\0code
C##\0\0\0\0\0\0CALL\0LIB$TRNLNM(...)
C#endif
.fi
.PP
When neither
.B _OS_UNIX
nor
.B _OS_VAXVMS
are defined, the output is
.PP
.nf
C#if\0_OS_UNIX
C##C\0\0\0\0\0UNIX\0code
C##\0\0\0\0\0\0	CALL\0GETENV(...)
C#elseif\0_OS_VAXVMS
C##C\0\0\0\0\0VMS\0code
C##\0\0\0\0\0\0CALL\0LIB$TRNLNM(...)
C#endif
.fi
.PP
When only
.B _OS_VAXVMS
is defined, the original input is sent to the output.
If by chance both
.B _OS_VAXVMS
and
.B _OS_UNIX
were defined, only the UNIX code would be selected,
because only the first branch of the conditional would
be executed.
.PP
Preprocessor conditionals may be nested:
.PP
.nf
.B "C\0#\0if\0_OS_UNIX"
.B "C\0\0\0#\0if\0_SUN"
.B "C\0\0\0\0\0\0#\0if\0_SUN4"
.B "C\0\0\0\0\0\0#\0elseif\0_SUN3"
.B "C\0\0\0\0\0\0#\0elseif\0_SUN386"
.B "C\0\0\0\0\0\0#\0endif"
.B "C\0\0\0#\0endif"
.B "C\0#\0elseif\0_OS_VMS"
.B "C\0#\0endif"
.fi
.PP
Any text following
.B #else
or
.B #endif
is ignored; it can be used to document the conditional, usually
with the test from the preceding
.BR #if :
.PP
.nf
.B "C\0#\0if\0_OS_UNIX"
.B "C\0#\0else\0NOT\0_OS_UNIX"
.B "C\0#\0endif\0_OS_UNIX"
.fi
.PP
.B fpp
directives are not executed if they are in a branch of a
conditional that is not currently selected.  However, conditional
statements are processed to keep track of the current nesting.
.PP
On UNIX, the
.B -Dname
option for
.BR diff (1)
can be used to get output from the comparison of two
files that is almost correct input for
.BR fpp .
A simple command pipe
.B "diff -Dxxx file1 file2 | sed -e 's/^#/C#/' >file3
will produce an output file
.I file3
from which
.B "fpp +Dxxx file3"
will recover 
.I file2
and 
.B "fpp +Uxxx file3"
will recover
.IR file1 .
.\"======================================================================
.SH "EXPRESSIONS"
Expressions are recognized and evaluated in two circumstances: in
the arguments of
.BR C#if ,
.BR C#elseif ,
and
.BR C#elif ,
and inside the parentheses of
.BR #(.\|.\|.) .
.PP
In expressions, primaries are Fortran integer, floating-point,
logical, and character constants, and preprocessor names.
.PP
Undefined names silently evaluate to zero in arithmetic
expressions, and to empty strings in string expressions.
.PP
Character strings appearing in arithmetic expressions are
converted to numbers, which are zero if the string does not look
like a number.  Character strings appearing by themselves
evaluate to themselves.
.PP
Arithmetic expressions are evaluated in
.I floating-point
arithmetic; for Boolean (Fortran logical) tests, zero is false,
and non-zero is true.
.PP
The usual Fortran arithmetic operators
.B +\0\-\0*\0/\0**
are recognized, along with the C modulus operator
.BR % ;
.BR "x % y" " is Fortran's " "mod(x,y)" .
This operator is rigorously defined for all arguments
to be
.BR "x % y = x - int(x/y)*y" .
.PP
The Fortran logical and relational operators are
supported, with convenient modern C-like synonyms:
.BR ".and. " ( &
.RB "and " && ),
.BR ".or. " ( |
.RB "and " || ),
.BR ".not. " ( ! ),
.BR ".eq. " ( == ),
.BR ".ne. " ( != ),
.BR ".lt. " ( < ),
.BR ".le. " ( <= ),
.BR ".gt. " ( > ),
and
.BR ".ge. " ( >= ).
Letter case in the dotted operators is not significant.
Finally, the Fortran character string concatenation
operator,
.BR "//" ,
is handled.
.PP
One special name,
.BR defined ,
is recognized, in any letter case; it may be used either in
functional form,
.BR defined(name) ,
or in prefix operator form,
.BR "defined name" .
It evaluates to 1 if the name is defined (even if the value of
the name is zero), and otherwise, to 0.  Several
.B defined
operators can be used in a single expression; that is much more
convenient than a series of nested conditionals using
.B C#ifdef
and
.BR C#ifndef .
.PP
The
.B #(.\|.\|.)
form is only recognized in a comment line, and the next line is
converted to a comment (see the section
.I "MACRO EXPANSION"
below); the parentheses hold an expression involving Fortran
constants and preprocessor names.
.PP
Examples of expressions are
.PP
.nf
.B "C#if\0\0defined(_OS_UNIX)\0\0||\0\0defined\0_OS_VAXVMS\0\0||\
\0\0(WORDSIZE\0==\032)"
.PP
.B "C\0\0\0\0\0REAL\0A(#(MAXA**2)),\0B(#(MAXA\0%\032))"
.PP
.B "C\0\0\0\0\0INTEGER\0BITS(#(WORDSIZE))"
.fi
.\"======================================================================
.SH "MACRO EXPANSION"
.B fpp
supports a
.I reversible
argument-free macro expansion capability.  This involves pairs of
lines, the first a comment line containing the macro references
as strings of the form
.BR #(constant-expression) ,
and the second a non-comment Fortran statement.
.PP
The first line of the pair is always exactly preserved in the
output, while the second is replaced by the expansion of the
comment, with the first character
.IR deleted ,
to change the comment into a non-comment.  The original contents
of the second line are preserved as a comment with the prefix
.B C-fpp-
in a third output line.
.PP
This peculiar input line pairing is necessary to ensure
that the expansion is reversible.
.PP
The parentheses around the expression serve to distinguish
between macros and
.B fpp
preprocessor directives in comments, and serendipitously permit
the extension from simple names to arbitrary constant expressions
that can be evaluated by
.BR fpp .
.PP
Care must be taken in writing the input to ensure that any
expected expansion does not make the line longer than 72
characters;
.B fpp
has almost no knowledge of Fortran, and therefore cannot provide
correct line wrapping for it.
.PP
Similarly, macro expansion in a multi-line continued statement
should avoided, since it introduces comment lines between
continuation lines.  While such comments are legal in full
Fortran 77, they are illegal in subset Fortran 77, and in older
Fortrans, and may cause problems for other tools that process
Fortran code.
.PP
Here is a small example.  Given command-line definitions
.PP
.nf
.B FPTYPE='DOUBLE PRECISION'
.B MAXA=19
.B MAXB=25
.fi
.PP
then the input
.PP
.nf
.B "C\0\0\0\0\0#(FPTYPE)\0A(#(MAXA)),\0B(#(MAXB),#(MAXA**2))"
.B "\0\0\0\0\0\0	REAL\0A(100),\0B(255,10000)"
.fi
.PP
is converted to the output
.PP
.nf
.B "C\0\0\0\0\0#(FPTYPE)\0A(#(MAXA)),\0B(#(MAXB),#(MAXA**2))"
.B "\0\0\0\0\0\0	DOUBLE\0PRECISION\0A(19),\0B(25,361)"
.B "C-fpp-\0\0\0\0\0\0REAL\0A(100),\0B(255,10000)"
.fi
.PP
Fortran 77
.B PARAMETER
statements can be used to achieve similar effects, but in more
restricted circumstances.  In particular,
.B fpp
permits such expansions to happen in strings:
.PP
.nf
.B "C10000 FORMAT ('Host operating system = #(OS)')"
.B "10000 FORMAT ('Host operating system = UNIX')"
.fi
.PP
This may be awkward to achieve in standard Fortran.
.\"======================================================================
.SH "MESSAGE OUTPUT STATEMENTS"
Text can be written to
.I stderr
with either of
.PP
.nf
.BR "C#message " text
.PP
.BR "C#error " text
.fi
.PP
The difference between them is that
.B "#error"
sets an exit code of 1, and also sends the text to
.IR stdout .
This can be used to ensure that a preprocessing error forces a
compilation error if an attempt is made later to compile the
output source program.
.PP
The output of both directives is prefixed with the file name
and input line number to identify the origin of the message.
.PP
When
.B "#error"
is executed, processing is not terminated; instead,
.B fpp
tries to process the remaining input so as to uncover additional
errors in the same run.
.\"======================================================================
.SH "OUTPUT OF fpp"
The output contains an initial comment header of the form
.PP
.nf
C-fpp- =================================================================
C-fpp- fpp version 1.0 [10-Dec-1990]
C-fpp- Date: Sat Dec  8 23:06:30 MST 1990
C-fpp- Directory: /u/sy/beebe
C-fpp- User: beebe@sandy.math.utah.edu
C-fpp- Macro: _OS_VAXVMS=1
C-fpp- Macro: FPTYPE=DOUBLE PRECISION
C-fpp- =================================================================
.fi
.PP
These comments provide a record of the processing, including what
symbol definitions and macro values have been selected.
.PP
Input comments beginning
.PP
.nf
.B "C-fpp-"
.fi
.PP
are flushed.  Thus, any existing comment header is always
replaced by a new header.
.PP
Each command-line
.B name=value
or
.B +Dname
setting, and each input definition directive
.PP
.nf
.B "C#define name value"
.fi
.PP
produce an output comment of the form
.PP
.nf
.B "C-fpp- Macro: name=value"
.fi
.PP
Thus, all output lines beginning
.PP
.nf
.B "C-fpp- Macro:"
.fi
.PP
document which names have been defined.  
.PP
A
.B C#undefine
statement results in output like
.PP
.nf
.B "C-fpp- Macro: name=--UNDEFINED--"
.fi
.\"======================================================================
.SH "PREDEFINED SYMBOLS"
Each implementation of
.B fpp
will predefine a few symbols that can be tested and used in
conditionals to select machine-specific code sections.  The
predefined symbols are provided to
.B fpp
ahead of any user-defined ones.  Since later redefinitions
override earlier ones, predefined symbols can always be changed
by the user.
.PP
Following ANSI C, predefined symbols always begin with two
underscores, or an underscore and an uppercase letter; such names
are reserved for the local implementation.
Predefined symbols that do not follow this convention are
.IR forbidden .
This requirement makes it possible to distinguish separate name
spaces for the user and for the implementation, preventing
surprises from unexpected substitutions that happen when code is
moved to a new environment.
.PP
The complete set of definitions is always recorded in the output
header; they can easily be displayed on
.I stdout
by giving
.B fpp
an empty input file:
.PP
.nf
.B "fpp /dev/null"
.fi
.PP
Following the conventions of the ANSI C preprocessor, predefined
symbols always
.I begin
with two underscores, or an underscore and an uppercase letter.
.PP
.B fpp
always predefines
.I "exactly one"
major operating-system symbol:
.PP
.nf
.B _OS_PCDOS
.B _OS_TOPS20
.B _OS_UNIX
.B _OS_VAXVMS
.fi
.PP
For
.BR _OS_UNIX ,
exactly one minor operating-system variant may also be defined:
.PP
.nf
.B _AIX
.B _AIX370
.B _BSD
.B _GOULD
.B _HPUX
.B _MACH
.B _MIPS
.B _POSIX
.B _STARDENT
.B _SUN
.fi
.PP
Additional architectural variants may be defined on some systems:
.PP
.nf
.B _IBM_3090
.B _IBM_PS_2
.B _IBM_RS_6000
.B _IBM_RT
.B _STARDENT_1500
.B _STARDENT_3000
.B _SUN3
.B _SUN386
.B _SUN4
.fi
Host byte addressing order is defined by one of
these:
.PP 
.nf 
.B _BIG_ENDIAN 
.B _LITTLE_ENDIAN 
.fi 
.PP
Big-endian addressing is used by IBM, Motorola,
and most RISC systems.  Little-endian addressing
is used by the Intel 80xxx and DEC VAX
architectures.
.PP
Host floating-point architecture must be defined by 
.nf
.B _IEEE_754
.fi
on those machines that have IEEE 754 floating-point arithmetic.
.PP
If the Fortran implementation supports NAMELIST I/O, the symbol
.nf
.B _NAMELIST
.fi
must be defined.
.PP
To ensure standardization, all such names must be registered
with the author of
.BR fpp ,
and will be listed in these manual pages.
.PP
Following ANSI C, four standard permanent symbols are always
defined; these each have two leading and two trailing
underscores.  Permanent symbols always begin with two
underscores, and they may not be redefined by the user.
.TP 13
.B _\|_DATE_\|_
Current calendar date in the form
.IR "Mmm dd yyyy" .
The month field is alphabetic, and the day number field has
a leading blank if the day is less than 10.
.TP
.B _\|_FILE_\|_
Current input file filename.
.TP
.B _\|_LINE_\|_
Current input file line number.
.TP
.B _\|_TIME_\|_
Wall-clock time in the form
.IR hh:mm:ss .
.TP
.B _\|_TIMEZONE_\|_
Three-letter time zone abbreviation, such as MDT for
Mountain Daylight Time, or GMT for Greenwich Mean Time.
.PP
The symbols
.BR _\|_DATE_\|_ ,
.BR _\|_TIME_\|_ ,
and
.B _\|_TIMEZONE_\|_
are set only once, at the start of execution of
.BR fpp .
These values can be conveniently used to generated output
stamped with the time of compilation:
.PP
.nf
C\0\0\0\0\0WRITE\0(*,*)\0'Compiled\0on\0#(__DATE__)\0at\0#(__TIME__)\0\
#(__TIMEZONE__)'
\0\0\0\0\0\0WRITE\0(*,*)\0'Compiled\0on\0???\0??\0????\0at\0??:??:??\0???'
.fi
.PP
might produce
.PP
.nf
C\0\0\0\0\0WRITE\0(*,*)\0'Compiled\0on\0#(__DATE__)\0at\0#(__TIME__)\0\
#(__TIMEZONE__)'
\0\0\0\0\0\0WRITE\0(*,*)\0'Compiled\0on\0Dec\010\01990\0at\009:10:07\0MST'
C-fpp-\0\0\0\0\0\0WRITE\0(*,*)\0'Compiled\0on\0???\0??\0????\0at\0??:??:??\
\0???'
.fi
.PP
.\"======================================================================
.SH "SEE ALSO"
.BR cc (1),
.BR cpp (1),
.BR diff (1),
.BR m4 (1),
.BR gawk (1),
.BR nawk (1).
.\"======================================================================
.SH "DIAGNOSTICS"
Diagnostics are issued to
.I stderr
if unclosed conditionals, or out-of-place conditional branches,
or errors in preprocessor expressions, are detected.  Attempts to
redefine permanent macros (any that begin with two underscores)
produce an error message.  Debug output requested by a
command-line option will be sent to
.IR stderr .
.PP
.nf
.BR C#error " text"
.fi
.PP
directives that are executed send their text argument to
.I stderr
and to
.IR stdout ,
and on UNIX, cause a later exit code of 1.
.PP
.nf
.BR C#message " text"
.fi
.PP
directives that are executed send their text argument to
.IR stderr .
.\"======================================================================
.SH "AUTHOR"
Nelson H. F. Beebe, Ph.D.
.br
Center for Scientific Computing
.br
Department of Mathematics
.br
South Physics Building
.br
University of Utah
.br
Salt Lake City, UT 84112
.br
Tel: (801) 581-5254
.br
FAX: (801) 581-4148
.br
Email: <beebe@science.utah.edu>
.\"==============================[The End]==============================
