/* Label Utility -- see labels.hlp for documentation
 
 This code is in the PUBLIC DOMAIN; bug reports and improvements are welcome!

 Author: Nelson H.F. Beebe (BEEBE@SCIENCE.UTAH.EDU),  Center for Scientific
         Computating, Dept. of Mathematics, University of Utah, Salt Lake City,
	 UT 84112

 [16-Jun-87] -- translated from SFTRAN3 to C
*/

#include <stdio.h>		/* for function declarations */
#include <ctype.h>		/* for isspace() only */
#include <values.h>		/* need for MAXLONG only */
#include <string.h>		/* for function declarations */

typedef int Boolean;
#define TRUE 1
#define FALSE 0

#define MAXADDRLINE 8		/* number of lines per address */
#define MAXWIDTH 50		/* address line width */
#define MAXLINE 80		/* line width */
#define LABELSIZE 9		/* number of lines per label */
#define IN(a,b,c) (((a) <= (b)) && ((b) <= (c)))

/* Global function definitions */
int  atoi();
long atol();

void check_date();
void classify_line();
void initialize();
void input_line();
void labels();
void print_label();
void store_line();
void store_header_line();
void store_spacer_lines();
char* strstr();
void trim_trailing_space();

/* Global data */
char address[MAXADDRLINE][MAXWIDTH]; /* address line buffer */
long after;			/* yymmdd after which labels are selected */
long before;			/* yymmdd before which labels are selected */
int copies;			/* number of copies of each label */
long date;			/* yymmdd of current address */
char header[MAXLINE];		/* label file header line */
char line[MAXLINE];		/* current input line */
int indent;			/* indentation on each label line */
int lc;				/* count of labels */
int repeat;			/* repeat input file this many times */

Boolean is_empty;		/* line is empty */
Boolean is_eofile;		/* end of file on input */
Boolean is_head;		/* have header line */
Boolean is_phone;		/* have phone line */
Boolean is_print;		/* have printable line for output labels */
Boolean is_remark;		/* have remark line */
Boolean is_title;		/* have title line */

int
main(argc,argv)
int argc;
char* argv[];
{
    initialize(argc,argv);
    labels();

    return (0);
}

void
labels()
{
    register int ktime;

    for (ktime = 1; ktime <= repeat; ktime++)
      {
	  input_line();
	  while (!is_eofile)
	    {
	      classify_line();
	      if (is_print)
		store_line();
	      else if (is_phone)
		check_date();
	      else if (is_empty)
		{
		  store_spacer_lines();
		  print_label();
		}
	      else if (is_title)
		store_header_line();
	      else if (is_remark)
		/* NO-OP */;
	      input_line();
	    }
	  store_spacer_lines();
	  print_label();
	  if (repeat > 1)
	      rewind (stdin);	/* !!! Must be real file--not pipe!!! */
      }
}

void
check_date()
{
    register int n;
    register char* p;
    register int dd;
    register int mm;
    register int yy;

    p = strstr(line,"Date:");
    if (p != (char*)NULL)
      {
	date = atol(p+5);	/* expect "Date: yymmdd" */
	if (date == 0L)
	  return;
	yy = (int)(date/10000L);
	if (yy >= 1900) yy -= 1900; /* allow 19yymmdd too */
	mm = (int)((date/100L) % 100L);
	dd = (int)(date % 100L);
	if (!(IN(70,yy,99) && IN(01,mm,12) && IN(01,dd,31)))
	  {			/* catch simple date errors (but not all) */
	    fprintf(stderr,"?Illegal DATE field [%s] in following address:\n",
		    line);
	    for (n = 0; n < lc; ++n)
		fputs(&address[n][0],stderr);
	  }
      }
}

void
classify_line()
{
  register int l;

  is_title = (line[0] == '=');
  is_remark = (line[0] == '*');
  is_print = !isspace(line[0]) && !(is_remark || is_title);
  is_empty = !(is_print || is_remark);

  for (l = 1; (is_empty && (l < MAXWIDTH) && line[l]); ++l)
      is_empty = isspace(line[l]);

  is_phone = !(is_title || is_print || is_remark || is_empty);
}

void
initialize(argc,argv)
int argc;
char* argv[];
{
  register int k;

  is_head = FALSE;
  lc = 0;
  date = 0L;
  after = -MAXLONG;
  before = MAXLONG;
  copies = 0;
  indent = 0;
  repeat = 1;

  for (k = 1; k < argc; ++k)
    {
      if (argv[k][0] == '-')
	switch (argv[k][1])
	{
	case 'a':
	case 'A':
	  after = atol(&argv[k][2]);
	  break;

	case 'b':
	case 'B':
	  before = atol(&argv[k][2]);
	  break;

	case 'c':
	case 'C':
	  copies = atoi(&argv[k][2]);
	  break;

	case 'i':
	case 'I':
	  indent = atoi(&argv[k][2]);
	  break;

	case 'r':
	case 'R':
	  repeat = atoi(&argv[k][2]);
	  break;

	default:
	  fprintf(stderr,"%s: Unknown switch [%s]\n",argv[0],argv[k]);
	  exit(1);
	}
      else
	{
	  fprintf(stderr,"%s: Argument not a switch [%s]\n",argv[0],argv[k]);
	  exit(1);
	}
    }
  if (after < 1L) after = -MAXLONG;
  if (before <= after) before = MAXLONG;
  if (copies < 1) copies = 1;
  if (indent < 1) indent = 0;
  if (repeat < 1) repeat = 1;
}

void
input_line()
{
    is_eofile = (fgets(line, MAXLINE, stdin) == (char*)NULL);
}

void
print_label()
{
    register int i;
    register int n;
    register int l;

    if ((lc > 0) && (date > after) && (date < before))
      {
	for (i = 1; i <= copies; ++i)
	  {
	    for (n = 0; n < MAXADDRLINE; ++n)
	      {
		for (l = 1; l <= indent; ++l)
		  putchar(' ');
		fputs(&address[n][0], stdout);
	      }
	    for (n = MAXADDRLINE+1; n <= LABELSIZE; ++n)
	      putchar('\n');	/* pad lines to end of label */
	  }
      }
    lc = 0;			/* address[][] is now empty */
    date = 0L;			/* and date is clear */
}

void
store_line()
{
    if (lc < MAXADDRLINE)
      {
	trim_trailing_space();
	strcpy(&address[lc][0],line);
      }
    lc++;
}

void
store_header_line()
{
    strcpy(header,&line[1]);
    is_head = TRUE;
}

void
store_spacer_lines()
{
    register int n;

    if (lc != 0)
      {
	for (n = lc; n < MAXADDRLINE; ++n)
	  {
	    address[n][0] = '\n';
	    address[n][1] = '\0';
	  }
      }
}

char*
strstr(s,t)		/* return address of t[] in s[], else (char*)NULL */
char* s;
char* t;
{
  register int k;
  register int l;

  l = strlen(t);
  for (k = strlen(s) - l; k >= 0; --k, ++s)
    {
      if (strncmp(s, t, l) == 0)
	return (s);
    }
  return ((char*)NULL);
}



void
trim_trailing_space()
{
    register int limit;

    limit = strlen(line);
    while ((limit >= 0) && (line[limit] == ' '))
      limit--;
    line[limit+1] == '\0';
}
