      SUBROUTINE SYSCMD (FILE,MAXFIL,NUMFIL,SWITCH,VALUE,MAXSW,NUMSW)
C     (COMMAND LINE FILE NAMES AND SWITCHES)
C     EXTRACT DECSYSTEM-20 FILE NAMES AND SWITCHES FROM THE
C     COMMAND LINE. THE NUMBER OF 10-CHARACTER FILE NAMES
C     RETURNED IN FILE(*) IS NUMFIL, UP TO A MAXIMUM OF "MAXFIL".
C     FILE NAMES ARE STORED IN PACKED CHARACTERS IN THE DOUBLE
C     PRECISION ARRAY, FILE(*), SO THAT AFTER PROVIDING DEFAULT
C     EXTENSIONS IF NECESSARY (WITH SYSEXT), THEY CAN BE USED TO
C     OPEN FILES (WITH SYSOPN).  THE SCAN TERMINATES AT
C     END-OF-LINE, OR AT THE FIRST SEMICOLON OR EXCLAMATION POINT
C     WHICH PRECEDES A COMMENT FIELD.
C
C     OPTION SWITCHES ARE IDENTIFIED BY LEADING SLASHES, AND
C     TERMINATE WITH WHITE SPACE.  WHITE SPACE IS CONSIDERED TO
C     BE BLANKS, COMMAS, AND TABS.  THE SWITCH NAMES ARE
C     EXTRACTED AND STORED WITHOUT THE SLASH IN THE DOUBLE
C     PRECISION ARRAY SWITCH(*), AND ANY INTEGER VALUE SEPARATED
C     FROM THE SWITCH BY A COLON OR EQUAL SIGN IS STORED IN
C     VALUE(*).  "NUMSW" SWITCHES AND VALUES, UP TO A MAXIMUM OF
C     "MAXSW", WILL BE RETURNED. IF NO INTEGER VALUE IS PROVIDED
C     FOR SWITCH K, THEN VALUE(K) IS SET TO 0. THUS, /SW, /SW:0,
C     AND /SW=0 ARE EQUIVALENT.
C
C     SAMPLE COMMAND LINES ARE:
C
C     @PRGNAM /A/B:34 FILE1.EXT, /C FILE2.EXT,FILE3.EXT ;COMMENT
C
C     @PRGNAM FILE1.EXT FILE2.EXT FILE3.EXT /A/B:34 /C
C
C     @PRGNAM /C/A/B:34 FILE1.EXT,FILE2.EXT,FILE3.EXT!EXAMPLE 3
C
C     IN EACH CASE, THE NAMES "FILE1.EXT", "FILE2.EXT", AND
C     "FILE3.EXT" WILL BE RETURNED IN FILE(*).  THE FILE
C     EXTENSIONS MAY BE OMITTED IF THE CALLING PROGRAM IS
C     PREPARED TO SUPPLY DEFAULTS WITH ROUTINE SYSEXT.
C     SWITCH(*) WILL CONTAIN "A", "B", AND "C", AND VALUE(*),
C     0, 34, AND 0, STORED IN THE ORDER THEY APPEARED ON THE
C     COMMAND LINE.
C
C     NO ATTEMPT IS MADE TO ELIMINATE DUPLICATE OCCURRENCES OF
C     SWITCHES. HOWEVER, IF THE SWITCH TABLE IS SCANNED IN
C     REVERSE ORDER, THEN THE FIRST MATCH WILL CORRESPOND TO THE
C     LAST OCCURRENCE OF A SWITCH ON THE COMMAND LINE.
C
C     IT IS PERMISSIBLE FOR FILE NAMES TO BEGIN WITH "@" (AT-SIGN),
C     SO THAT INDIRECT COMMAND FILES CAN BE SPECIFIED.  THE "@"
C     WILL BE INCLUDED AS THE FIRST CHARACTER OF THE FILE NAME.
C
C     THE SYNTAX CHECKING IN THIS ROUTINE IS LAZY.  WHENEVER AN
C     UNEXPECTED CHARACTER IS FOUND, THE ACTION IS SIMPLY TO
C     TERMINATE THE SCAN FOR THE CURRENT TOKEN.  NO ATTEMPT IS
C     MADE TO VERIFY THAT A FILE NAME ADHERES TO THE OPERATING
C     SYSTEM CONVENTIONS.  IT IS ONLY REQUIRED TO BE MADE UP OF
C     LETTERS, DIGITS, AND PERIODS (OF WHICH THERE COULD BE MORE
C     THAN ONE), AND TO BEGIN WITH A LETTER.  "@" IS CONSIDERED
C     TO BE A LETTER, SO A TOKEN "@@..@@" COULD BE RETURNED AS
C     A FILE NAME, EVEN THOUGH IT IS NOT A VALID ONE.
C     (03-JUL-80)
      INTEGER          ASCII,       CHAR,        CMDLIN(25),  DECVAL
      INTEGER          IASC,        IASC1,       IASC2,       KARASC
      INTEGER          KARUC,       LAST,        LENCMD,      LENTOK
      INTEGER          MAXCMD,      MAXFIL,      MAXLEN,      MAXSW
      INTEGER          NUMBER,      NUMFIL,      NUMSW
      INTEGER          VALUE(MAXSW)
      DOUBLE PRECISION BLANK,       FILE(MAXFIL),SWITCH(MAXSW)
      LOGICAL          ASSIGN,      AVAIL,       DIGIT,       DOT
      LOGICAL          FILECH,      FIRST,       IN,          MINUS
      LOGICAL          PLUS,        POSVAL,      REMARK,      SLASH
      LOGICAL          UCLETT,      WHITE
      DATA FIRST/.TRUE./, MAXLEN/10/, BLANK/1H /, MAXCMD/25/
      ASSIGN(IASC) = (IASC .EQ. 58) .OR. (IASC .EQ. 61)
      IN(IASC1,IASC,IASC2) = (IASC1 .LE. IASC) .AND. (IASC .LE. IASC2)
      DECVAL(IASC) = IASC - 48
      DIGIT(IASC) = IN(48,IASC,57)
      DOT(IASC) = (IASC .EQ. 46)
      REMARK(IASC) = (IASC .EQ. 59) .OR. (IASC .EQ. 33)
      SLASH(IASC) = (IASC .EQ. 47)
      UCLETT(IASC) = IN(64,IASC,90)
      FILECH(IASC) = UCLETT(IASC) .OR. DIGIT(IASC) .OR. DOT(IASC)
      MINUS(IASC) = (IASC .EQ. 45)
      PLUS(IASC) = (IASC .EQ. 43)
      WHITE(IASC) = (IASC .EQ. 32) .OR. (IASC .EQ. 44) .OR.
     X(IASC .EQ. 9)
      ASSIGN 20001 TO NPR001
      GO TO 30001
20001 IF (.NOT.(MAXFIL .GT. 0)) GO TO 20002
      LAST = 0
      ASSIGN 20005 TO NPR002
      GO TO 30002
20005 GO TO 20007
20006 IF ((LENTOK .EQ. 0) .OR. (NUMFIL .EQ. MAXFIL)) GO TO 20008
20007 ASSIGN 20009 TO NPR003
      GO TO 30003
20009 IF (LENTOK .GT. 0) NUMFIL = NUMFIL + 1
      GO TO 20006
20008 CONTINUE
20002 CONTINUE
20003 IF (.NOT.(MAXSW .GT. 0)) GO TO 20010
      LAST = 0
      ASSIGN 20013 TO NPR002
      GO TO 30002
20013 GO TO 20015
20014 IF ((LENSW .EQ. 0) .OR. (NUMSW .EQ. MAXSW)) GO TO 20016
20015 ASSIGN 20017 TO NPR004
      GO TO 30004
20017 IF (LENSW .GT. 0) NUMSW = NUMSW + 1
      GO TO 20014
20016 CONTINUE
20010 RETURN
C
C-----------------------------------------------------------------------
C---- PROCEDURE (INITIALIZE)
30001 NUMFIL = 0
      NUMSW = 0
      IF (.NOT.(FIRST)) GO TO 20018
      LENCMD = 0
      CALL SYSGCL (CMDLIN,MAXCMD,LENCMD)
20018 IF (LENCMD .LE. 0) RETURN
      FIRST = .FALSE.
      GO TO 20001
C
C-----------------------------------------------------------------------
C---- PROCEDURE (GET FILE TOKEN)
30003 LENTOK = 0
      FILE(NUMFIL+1) = BLANK
      ASSIGN 20021 TO NPR005
      GO TO 30005
20021 IF (.NOT.(AVAIL)) GO TO 20022
20025 IF (.NOT.(AVAIL .AND. .NOT.UCLETT(ASCII))) GO TO 20026
      ASSIGN 20027 TO NPR002
      GO TO 30002
20027 GO TO 20025
20026 CONTINUE
20028 IF (.NOT.(AVAIL .AND. FILECH(ASCII))) GO TO 20029
      LENTOK = LENTOK + 1
      IF (LENTOK .LE. MAXLEN) CALL KARPUT (CHAR,
     XFILE(NUMFIL+1),LENTOK)
      ASSIGN 20030 TO NPR002
      GO TO 30002
20030 GO TO 20028
20029 ASSIGN 20031 TO NPR006
      GO TO 30006
20031 CONTINUE
20022 GO TO 20009
C
C-----------------------------------------------------------------------
C---- PROCEDURE (GET SWITCH TOKEN)
30004 LENSW = 0
      VALUE(NUMSW+1) = 0
      SWITCH(NUMSW+1) = BLANK
20032 IF (.NOT.(AVAIL .AND. .NOT.SLASH(ASCII))) GO TO 20033
      ASSIGN 20034 TO NPR002
      GO TO 30002
20034 GO TO 20032
20033 ASSIGN 20035 TO NPR002
      GO TO 30002
20035 CONTINUE
20036 IF (.NOT.(AVAIL .AND. .NOT.WHITE(ASCII) .AND.
     X.NOT.SLASH(ASCII))) GO TO 20037
      IF (.NOT.(.NOT.ASSIGN(ASCII))) GO TO 20038
      LENSW = LENSW + 1
      IF (LENSW .LE. MAXLEN) CALL KARPUT (CHAR,
     XSWITCH(NUMSW+1),LENSW)
      ASSIGN 20041 TO NPR002
      GO TO 30002
20041 GO TO 20039
20038 ASSIGN 20042 TO NPR002
      GO TO 30002
20042 ASSIGN 20043 TO NPR007
      GO TO 30007
20043 VALUE(NUMSW+1) = NUMBER
20039 GO TO 20036
20037 GO TO 20017
C
C-----------------------------------------------------------------------
C---- PROCEDURE (EXTRACT NUMERIC VALUE)
30007 NUMBER = 0
      POSVAL = .TRUE.
      IF (.NOT.(PLUS(ASCII))) GO TO 20044
      ASSIGN 20047 TO NPR002
      GO TO 30002
20047 GO TO 20045
20044 IF (.NOT.(MINUS(ASCII))) GO TO 10001
      ASSIGN 20048 TO NPR002
      GO TO 30002
20048 POSVAL = .FALSE.
10001 CONTINUE
20045 CONTINUE
20049 IF (.NOT.(AVAIL .AND. DIGIT(ASCII))) GO TO 20050
      NUMBER = 10*NUMBER - DECVAL(ASCII)
      ASSIGN 20051 TO NPR002
      GO TO 30002
20051 GO TO 20049
20050 IF (POSVAL) NUMBER = -NUMBER
      GO TO 20043
C
C-----------------------------------------------------------------------
C---- PROCEDURE (FLUSH SWITCHES AND WHITE SPACE)
30005 GO TO 20053
20052 IF (.NOT.AVAIL .OR. .NOT.SLASH(ASCII)) GO TO 20054
20053 IF (.NOT.(SLASH(ASCII))) GO TO 20055
      ASSIGN 20055 TO NPR006
      GO TO 30006
20055 CONTINUE
20056 IF (.NOT.(AVAIL .AND. WHITE(ASCII))) GO TO 20057
      ASSIGN 20058 TO NPR002
      GO TO 30002
20058 GO TO 20056
20057 GO TO 20052
20054 GO TO 20021
C
C-----------------------------------------------------------------------
C---- PROCEDURE (ADVANCE TO WHITE SPACE)
30006 CONTINUE
20059 IF (.NOT.(AVAIL .AND. .NOT.WHITE(ASCII))) GO TO 20060
      ASSIGN 20061 TO NPR002
      GO TO 30002
20061 GO TO 20059
20060 GO TO NPR006, (20031,20055)
C
C-----------------------------------------------------------------------
C---- PROCEDURE (GET NEXT CHARACTER)
30002 LAST = LAST + 1
      CALL KARGET (CHAR,CMDLIN,LAST)
      CHAR = KARUC(CHAR)
      ASCII = KARASC(CHAR)
      AVAIL = (LAST .LE. LENCMD) .AND. .NOT.REMARK(ASCII)
      GO TO NPR002, (20005,20013,20027,20030,20034,20035,20041,20042,200
     X47,20048,20051,20058,20061)
C
      END
